@@ -80,6 +80,9 @@ def makeGenOpts(args):
     # Output target directory
     directory = args.directory
 
+    # Path to generated files, particularly api.py
+    genpath = args.genpath
+
     # Descriptive names for various regexp patterns used to select
     # versions and extensions
     allFeatures = allExtensions = '.*'
@@ -133,6 +136,7 @@ def makeGenOpts(args):
             conventions=conventions,
             filename='vk_typemap_helper.h',
             directory=directory,
+            genpath=None,
             apiname='vulkan',
             profile=None,
             versions=featuresPat,
@@ -158,6 +162,7 @@ def makeGenOpts(args):
             conventions=conventions,
             filename='mock_icd.h',
             directory=directory,
+            genpath=None,
             apiname='vulkan',
             profile=None,
             versions=featuresPat,
@@ -183,6 +188,7 @@ def makeGenOpts(args):
             conventions=conventions,
             filename='mock_icd.cpp',
             directory=directory,
+            genpath=None,
             apiname='vulkan',
             profile=None,
             versions=featuresPat,
@@ -208,6 +214,7 @@ def makeGenOpts(args):
             conventions=conventions,
             filename='vulkaninfo.hpp',
             directory=directory,
+            genpath=None,
             apiname='vulkan',
             profile=None,
             versions=featuresPat,
@@ -309,7 +316,9 @@ if __name__ == '__main__':
     parser.add_argument('-time', action='store_true',
                         help='Enable timing')
     parser.add_argument('-validate', action='store_true',
-                        help='Enable group validation')
+                        help='Enable XML group validation')
+    parser.add_argument('-genpath', action='store', default='gen',
+                        help='Path to generated files')
     parser.add_argument('-o', action='store', dest='directory',
                         default='.',
                         help='Create target and related files in specified directory')
